/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui.transactions

import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.Scaffold
import androidx.compose.material3.SnackbarDuration
import androidx.compose.material3.SnackbarHost
import androidx.compose.material3.SnackbarHostState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.remember
import androidx.navigation.NavOptions
import de.chaosdorf.meteroid.ui.navigation.HomeSections
import de.chaosdorf.meteroid.ui.navigation.MeteroidBottomBar
import de.chaosdorf.meteroid.ui.navigation.MeteroidTopBar
import kotlinx.datetime.Clock
import kotlinx.datetime.TimeZone
import kotlinx.datetime.toLocalDateTime
import java.time.Month

@Composable
fun TransactionListScreen(
  viewModel: TransactionViewModel,
  onNavigate: (String, NavOptions) -> Unit
) {
  val account by viewModel.account.collectAsState()
  val transactions by viewModel.transactions.collectAsState()
  val snackbarHostState = remember { SnackbarHostState() }

  LaunchedEffect(account) {
    val offline = viewModel.checkOffline(account?.server)
    snackbarHostState.currentSnackbarData?.dismiss()
    if (offline) {
      snackbarHostState.showSnackbar(
        message = "Unable to connect to server",
        duration = SnackbarDuration.Indefinite
      )
    }
  }

  Scaffold(
    topBar = { MeteroidTopBar(account, onNavigate, viewModel::togglePin) },
    bottomBar = {
      MeteroidBottomBar(
        currentRoute = HomeSections.HISTORY,
        historyEnabled = account?.user?.audit == true,
        wrappedEnabled = Clock.System.now().toLocalDateTime(TimeZone.currentSystemDefault())
          .month.let { it == Month.NOVEMBER || it == Month.DECEMBER },
        navigateTo = onNavigate
      )
    },
    snackbarHost = {
      SnackbarHost(hostState = snackbarHostState)
    }
  ) { paddingValues: PaddingValues ->
    LazyColumn(contentPadding = paddingValues) {
      items(transactions) { (transaction, drink) ->
        TransactionListItem(transaction, drink)
      }
    }
  }
}
